
:- style_check(-singleton).

/********************************* IM POTP *************************************/

/* There are 3 types of POTP,
which are POTP1410, POTP1460, POTP1485. Here we describe only 1460 for example*/

/******************************************************************************/
/***************************** POTP 1460 ***************************************/
/******************************************************************************/

% potp(name, numOfSlots, supported_cards[])
potp('POTP1460',20, supported_cards(['SM-8xSTM', 'SM-6xE3/T3', 'SM-1x10GE-20xGE-B', 'SM-10xGE-32xMAP-B',
'SM-1xS64-4xS16-AH', 'SM-2x10GE-10xGE-A', 'SM-4xXFP-2xSFP+', 'SM-2xXFP-16xSFP',
'SM-2x10GE-10xGE-B', 'SM-1x10GE-20xGE-C', 'SM-10xGE-32xMAP-C','SC-BH', 'SC-AH'])).

/************************** MODULE MODES OF CARDS *****************************/

/* Supported module modes for each card.
NOTE: Module mode is nothing more than a subset of available ports that are
activated. For instance, when we see the name of the card:
'SM-10xGE-32xMAP-B'
that means we have 10 GE ports and 32 MAP-B ports, and so possible modules modes
are:
'10xGE(SM) 32xMAP(SM) A' <--- we activated all ports for use
and
'10xGE(SM) B' <--- we activated just 10 GE ports
*/


% card_type( CARD, supported_mms['MM1', ... 'MMn'] )


card_type('SC-AT', supported_mms(['SC-AT', 'SCM'])).
card_type('SC-AH', supported_mms(['SC', 'SCM'])).
card_type('SC-BH', supported_mms(['SC-BH'])).
card_type('SM-8xSTM', supported_mms(['8xSTM(SM)'])).
card_type('SM-1x10GE-20xGE-B', supported_mms(['10xGE(SM) A'])).
card_type('SM-10xGE-32xMAP-B', supported_mms(['10xGE(SM) 32xMAP(SM) A', '10xGE(SM) B'])).
card_type('SM-1xS64-4xS16-AH', supported_mms(['1xSTM64/OTU2(SM)', '4xSTM16/OTU1(SM)', '2xSTM16/OTU1(SM)'])).
card_type('SM-1xS64-4xS16-A', supported_mms(['1xSTM64/OTU2(SM)'])).
card_type('SM-4xXFP-2xSFP+', supported_mms(['4xXFP(SM)', '4xXFP OTU2e 10xGE (SM)'])).
card_type('SM-2xXFP-16xSFP', supported_mms(['2xXFP 6xSFP(SM)', '6xSFP(SM)'])).
card_type('SM-2x10GE-10xGE-A', supported_mms(['2x10GE(SM) A'])).
card_type('SM-2x10GE-10xGE-B', supported_mms(['2x10GE(SM) 2xGE(SM)'])).
card_type('SM-1x10GE-20xGE-C', supported_mms(['10xGE(SM) C'])).
card_type('SM-10xGE-32xMAP-C', supported_mms(['10xGE(SM) 32xMAP(SM) A', '10xGE(SM) B'])).
card_type('DF-AH', supported_mms(['DF'])).
card_type('SM-6xE3/T3', supported_mms(['6xE3/T3(EM)'])).
card_type('SM-63xE1',  supported_mms(['63xE1(EM)'])).

/****************************** POSSIBLE PORTS ********************************/

/* Description of port types

NOTE: Each port can be switchable or not-switchable, for instance OTN ports can
be switched to CBR, but only on some module modes. Each port type for a specific
module mode has a subset of possible attributes, depending on port type. Depending
on a possible port attributes we can define possible pluggable modules (below).

*/

/*   card_type( MM1, port_types[

   (PORT1,
   [
   (PORT_TYPE_1, possible_configs[TYPE1, TYPE2, ... , TYPEn]),
   (PORT_TYPE_2, possible_configs[TYPE1, TYPE2, ... , TYPEm])
   ]),
   
   (PORT2,
   [
   (PORT_TYPE_1, possible_configs[TYPE1, TYPE2, ... , TYPEn]),
   (PORT_TYPE_2, possible_configs[TYPE1, TYPE2, ... , TYPEm])
   ]),

   ...
   
   (PORTX,
   [
   (PORT_TYPE_1, possible_configs[TYPE1, TYPE2, ... , TYPEn]),
   (PORT_TYPE_2, possible_configs[TYPE1, TYPE2, ... , TYPEm])
   ])
   
   ])

*/

% module mode 4xXFP(SM)

/* NOTE: For instance, module mode 4xXFP(SM) has 4 ports, each of them is
switchable, OTN ports can be of KLEVEL 2 and KLEVEL 3 and CBR ports can be of
four types. */

% era ports_and_modules
% era possible_type
module_mode('4xXFP(SM)', port_types([
(1, [
('OTN', possible_configs(['klevel2', 'klevel2e'])),
('CBR', possible_configs(['lan 10g', 'oc192', 'stm64', 'wan 10g']))]),
(2, [
('OTN', possible_configs(['klevel2', 'klevel2e'])),
('CBR', possible_configs(['lan 10g', 'oc192', 'stm64', 'wan 10g']))]),
(3, [
('OTN', possible_configs(['klevel2', 'klevel2e'])),
('CBR', possible_configs(['lan 10g', 'oc192', 'stm64', 'wan 10g']))]),
(4, [
('OTN', possible_configs(['klevel2', 'klevel2e'])),
('CBR', possible_configs(['lan 10g', 'oc192', 'stm64', 'wan 10g']))])
])).

% module mode 4xXFP OTU2e 10xGE (SM)

module_mode('4xXFP OTU2e 10xGE (SM)', port_types([
(1, [
('OTN', possible_configs(['klevel2', 'klevel2e']))]),
(2, [
('OTN', possible_configs(['klevel2', 'klevel2e']))]),
(3, [
('OTN', possible_configs(['klevel2', 'klevel2e']))]),
(4, [
('OTN', possible_configs(['klevel2', 'klevel2e']))])
])).


/********************** POSSIBLE PLUGGABLE MODULES ****************************/

/* Description of possible pluggable modules for each port type, depending on an
port attribute */

% possible_port_config(Attribute, [ PossiblePluggableModule1, ... PossiblePluggableModuleN])

possible_port_config('klevel1', [
'S-OTU1.1',
'L-OTU1.1',
'L-OTU1.2',
'L-OTU1.2 CWDM 28 DB',
'L-OTU1.2 DWDM']).
possible_port_config('klevel2', [
'OTU-2 P1L1-2D1F',
'OTU-2 ZX',
'OTU-2 EX',
'OTU-2 DWDM',
'OTU-2/2e S DWDM',
'OTU-2/2e CWDM']).
possible_port_config('klevel2e', [
'OTU-2e P1L1-2D1F',
'OTU-2e ZX',
'OTU-2e EX',
'OTU-2e DWDM',
'OTU-2/2e S DWDM',
'OTU-2/2e CWDM']).
possible_port_config('klevel4', [
'OTU-4 DWDM']).

/******************************************************************************/

possible_port_config('lan 1g',
['1000BASE-T',
'1000BASE-ZX',
'1000BASE-LX 10',
'1000BASE-SX',
'1000BASE CWDM 32DB',
'1000BASE CWDM 28DB',
'1000BASE-BX 10-U',
'1000BASE-BX 10-D',
'1000BASE DWDM',
'1000BASE-BX80-U',
'1000BASE-BX80-D',
'1000BASE-SX-RMON',
'1000BASE-LX10-RMON']).
possible_port_config('lan 1g sync e',
['1000BASE-T',
'1000BASE-ZX',
'1000BASE-LX 10',
'1000BASE-SX',
'1000BASE CWDM 32DB',
'1000BASE CWDM 28DB',
'1000BASE-BX 10-U',
'1000BASE-BX 10-D',
'1000BASE DWDM',
'1000BASE-BX80-U',
'1000BASE-BX80-D',
'1000BASE-SX-RMON',
'1000BASE-LX10-RMON']).
possible_port_config('lan 10g',
['10GBASE-LR/LW',
'10GBASE-ER/EW',
'10GBASE-ZR/ZW',
'10000BASE-SX',
'10000BASE-LH',
'10GBASE DWDM',
'10GBASE CWDM']).
possible_port_config('oc48',
['S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-16.2 CWDM 28DB',
'L-16.2 DWDM',
'L-16_SFW_D_26DB',
'L-16_SFW_U_26DB']).
possible_port_config('oc192',
['I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM',
'S-64.2 DWDM',
'L-64.2 CWDM']).
possible_port_config('stm16',
['S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-16.2 CWDM 28DB',
'L-16.2 DWDM',
'L-16_SFW_D_26DB',
'L-16_SFW_U_26DB']).
possible_port_config('stm64',
['I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM',
'S-64.2 DWDM',
'L-64.2 CWDM']).
possible_port_config('wan 10g',
['I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM',
'S-64.2 DWDM',
'L-64.2 CWDM']).

/******************************************************************************/

possible_port_config('STM1',
['S-1.1',
'L-1.1',
'S-1.1E',
'L-1.2',
'S-4.1',
'L-4.1',
'L-4.2',
'L-4.2 CWDM 32DB',
'S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-1.2 CWDM 28DB',
'L-1.2 CWDM 32DB',
'L-4.2 CWDM 28DB',
'L-16.2 CWDM 28DB',
'STM-1 (SFW-D)',
'STM-1 (SFW-U)',
'STM-4 (SFW-D)',
'STM-4 (SFW-U)',
'L-1.2 DWDM',
'L-4.2 DWDM',
'L-16.2 DWDM',
'I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM' ,
'L_1_SFW_D_26DB',
'L_1_SFW_U_26DB',
'L_4_SFW_D_26DB',
'L_4_SFW_U_26DB',
'L_16_SFW_D_26DB',
'L_16_SFW_U_26DB',
'S-64.2 DWDM',
'L-64.2 CWDM',
'S-64.2b',
'L-64.2']).
possible_port_config('STM4',
['S-1.1',
'L-1.1',
'S-1.1E',
'L-1.2',
'S-4.1',
'L-4.1',
'L-4.2',
'L-4.2 CWDM 32DB',
'S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-1.2 CWDM 28DB',
'L-1.2 CWDM 32DB',
'L-4.2 CWDM 28DB',
'L-16.2 CWDM 28DB',
'STM-1 (SFW-D)',
'STM-1 (SFW-U)',
'STM-4 (SFW-D)',
'STM-4 (SFW-U)',
'L-1.2 DWDM',
'L-4.2 DWDM',
'L-16.2 DWDM',
'I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM' ,
'L_1_SFW_D_26DB',
'L_1_SFW_U_26DB',
'L_4_SFW_D_26DB',
'L_4_SFW_U_26DB',
'L_16_SFW_D_26DB',
'L_16_SFW_U_26DB',
'S-64.2 DWDM',
'L-64.2 CWDM',
'S-64.2b',
'L-64.2']).
possible_port_config('STM16',
['S-1.1',
'L-1.1',
'S-1.1E',
'L-1.2',
'S-4.1',
'L-4.1',
'L-4.2',
'L-4.2 CWDM 32DB',
'S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-1.2 CWDM 28DB',
'L-1.2 CWDM 32DB',
'L-4.2 CWDM 28DB',
'L-16.2 CWDM 28DB',
'STM-1 (SFW-D)',
'STM-1 (SFW-U)',
'STM-4 (SFW-D)',
'STM-4 (SFW-U)',
'L-1.2 DWDM',
'L-4.2 DWDM',
'L-16.2 DWDM',
'I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM' ,
'L_1_SFW_D_26DB',
'L_1_SFW_U_26DB',
'L_4_SFW_D_26DB',
'L_4_SFW_U_26DB',
'L_16_SFW_D_26DB',
'L_16_SFW_U_26DB',
'S-64.2 DWDM',
'L-64.2 CWDM',
'S-64.2b',
'L-64.2']).
possible_port_config('STM64',
['S-1.1',
'L-1.1',
'S-1.1E',
'L-1.2',
'S-4.1',
'L-4.1',
'L-4.2',
'L-4.2 CWDM 32DB',
'S-16.1',
'L-16.1',
'L-16.2',
'L-16.2 CWDM 32DB',
'L-1.2 CWDM 28DB',
'L-1.2 CWDM 32DB',
'L-4.2 CWDM 28DB',
'L-16.2 CWDM 28DB',
'STM-1 (SFW-D)',
'STM-1 (SFW-U)',
'STM-4 (SFW-D)',
'STM-4 (SFW-U)',
'L-1.2 DWDM',
'L-4.2 DWDM',
'L-16.2 DWDM',
'I-64.1',
'S-64.2',
'L-64.2',
'L-64.2 DWDM' ,
'L_1_SFW_D_26DB',
'L_1_SFW_U_26DB',
'L_4_SFW_D_26DB',
'L_4_SFW_U_26DB',
'L_16_SFW_D_26DB',
'L_16_SFW_U_26DB',
'S-64.2 DWDM',
'L-64.2 CWDM',
'S-64.2b',
'L-64.2']).

/******************************************************************************/

possible_port_config('L1',
['10GBASE-LR/LW',
'10GBASE-ER/EW',
'10GBASE-ZR/ZW',
'10000BASE-SX',
'10GBASE DWDM',
'10GBASE CWDM']).
possible_port_config('L1 VLAN MUX',
['10GBASE-LR/LW',
'10GBASE-ER/EW',
'10GBASE-ZR/ZW',
'10000BASE-SX',
'10GBASE DWDM',
'10GBASE CWDM']).
possible_port_config('L2',
['10GBASE-LR/LW',
'10GBASE-ER/EW',
'10GBASE-ZR/ZW',
'10000BASE-SX',
'10GBASE DWDM',
'10GBASE CWDM']).
possible_port_config('MPLS', ['10GBASE-LR/LW',
'10GBASE-ER/EW',
'10GBASE-ZR/ZW',
'10000BASE-SX',
'10GBASE DWDM',
'10GBASE CWDM']).

