:- style_check(-singleton).

can_run_on(MCTP, PCTP) :-

    nl,
    pctp(PCTP, potp(PCTP_POTPS), instrument(PCTP_INSTRUMENTS), au2(PCTP_AU2S), au1(PCTP_AU1S)),
    mctp(MCTP, potp(MCTP_POTPS), instrument(MCTP_INSTRUMENTS), au2(MCTP_AU2S), au1(MCTP_AU1S)),
    compliance_INSTRUMENTS(MCTP_INSTRUMENTS, PCTP_INSTRUMENTS),
    compliance_AU1S(MCTP_AU1S, PCTP_AU1S),
    compliance_AU2S(MCTP_AU2S, PCTP_AU2S),

    bagof((Res,Eff),
       (compliance_potps(MCTP_POTPS, PCTP_POTPS, Res), calculate_efficiency(Res, Eff)),
       Total_result),
    find_optimal_mapping_number(Total_result, Answer),
    print_mapping(Total_result),
    write('*****************************************'),nl,nl,
    write('Best candidate(s) for mapping is(are):'),nl,nl,
    find_best_mapping(Total_result, Answer),
    write('*****************************************'),nl,nl,
    write('Success.'),nl,nl,
    write_to_file('Success.').

write_to_file(What) :-
open('abc.txt', append, Stream),
write(Stream, What),
nl(Stream),
close(Stream).

write_to_file(A, B, C) :-
open('abc.txt', append, Stream),
write(Stream, A),
write(Stream, B),
write(Stream, C),
nl(Stream),
close(Stream).



calculate_efficiency([], 0).
calculate_efficiency([(POTP_A, POTP_B)|T], Eff) :-
    potp_instance(POTP_A, attributes([_, _, _, ('INSTALLED_CARDS', CARDS_MCTP)])),
    potp_instance(POTP_B, attributes([_, _, _, ('INSTALLED_CARDS', CARDS_PCTP)])),
    length(CARDS_MCTP, Length_MCTP),
    length(CARDS_PCTP, Length_PCTP),
    K is Length_PCTP-Length_MCTP,
    calculate_efficiency(T, Rest),
    Eff is K+Rest.


smaller(X, Y, X) :- (X =< Y).
smaller(X, Y, Y) :- (Y < X).
find_optimal_mapping_number([(Mapping, Efficiency)|[]], Efficiency).
find_optimal_mapping_number([(Mapping, Efficiency)|Tail], Answer) :-
    find_optimal_mapping_number(Tail, What),
    smaller(What, Efficiency, Answer).

find_best_mapping(Total_result, Answer) :-
    foreach( (member((List, Eff), Total_result), Eff = Answer) , (
        foreach((member((A,B), List)),(write(A),write(' > '),write(B),nl)),nl
    ) ).

print_mapping(Total_result) :-
    foreach( (member((List, Eff), Total_result)) , (
    write('Mapping'),write(' (efficiency is '),write(Eff),write('): '),nl,nl,foreach((member((A,B), List)),(write(A),write(' > '),write(B),nl)),nl,
    write_to_file('Mapping: '),foreach((member((A,B), List)),(write_to_file(A,' > ',B)))
    ) ).


% compliance_potps(  ['POTP1', 'POTP2', 'POTP3'],   ['ne1', 'ne2', 'ne21', 'ne4']  , R).

compliance_potps([], _, []).
compliance_potps([H|T], List, Res) :-
    compliance_card_aux(H,List,(H,PCTP_POTP)),
    delete(List, PCTP_POTP, Removed),
    compliance_potps(T,Removed,Sol),
    append([(H,PCTP_POTP)],Sol,Res).


compliance_card_aux(POTP,[Head|Tail],(POTP,Head)) :-
    compliance_cards(POTP,Head,(POTP,Head)).
compliance_card_aux(POTP,[Head|Tail],R) :-
    compliance_card_aux(POTP,Tail,R).

% for example ('POTP1','ne1')
compliance_cards(Id_MCTP, Id_PCTP, (Id_MCTP, Id_PCTP)) :-

     potp_instance(Id_MCTP, attributes([TYPE_MCTP,
                              IP_ADDR_MCTP,
                              NTP_ADDR_MCTP,
                              ('INSTALLED_CARDS', CARDS_MCTP)])),


     potp_instance(Id_PCTP, attributes([TYPE_PCTP,
                                       IP_ADDR_PCTP,
                                       NTP_ADDR_PCTP,
                                       ('INSTALLED_CARDS', CARDS_PCTP)])),

     cards(CARDS_MCTP, CARDS_PCTP, Result).



cards([], _, []).
cards([Single_MCTP_Card|T], PCTP_List, Res) :-
    cards_aux(Single_MCTP_Card, PCTP_List, (Single_MCTP_Card, Single_PCTP_Card)),
    delete(PCTP_List, Single_PCTP_Card, Removed),
    cards(T, Removed, Sol),
    append([(Single_MCTP_Card, Single_PCTP_Card)], Sol, Res).


cards_aux(Single_MCTP_Card,[Single_PCTP_Card|T],(Single_MCTP_Card, Single_PCTP_Card)) :-
    two_cards(Single_MCTP_Card, Single_PCTP_Card, (Single_MCTP_Card, Single_PCTP_Card)),!.

cards_aux(Single_MCTP_Card,[Single_PCTP_Card|T],R) :-
    cards_aux(Single_MCTP_Card,T,R).



two_cards(
slot(_,card(MODULE_MODE),installed_interface(MCTP_INTERFACES)),
slot(_,card(CARD_TYPE),installed_interface(PCTP_INTERFACES)),
(slot(_,card(MODULE_MODE),installed_interface(MCTP_INTERFACES)),
slot(_,card(CARD_TYPE),installed_interface(PCTP_INTERFACES)))) :-

MCTP_INTERFACES = PCTP_INTERFACES,
card_type(CARD_TYPE, supported_mms(SUPPORTED_MODULE_MODES)),
member(MODULE_MODE, SUPPORTED_MODULE_MODES).


check_length(A,B,ElemType) :-

    length(A, A_Length),
    length(B, B_Length),
    ignore((A_Length @> B_Length) ->  (write('Not enough '),write(ElemType))),
    A_Length @=< B_Length.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

compliance_AU2S(MCTP_AU2S, PCTP_AU2S) :-
    check_length(MCTP_AU2S,PCTP_AU2S,'AU2s'),
    au2s(MCTP_AU2S, PCTP_AU2S, R),
    write('AU2s OK.'),nl,nl,
    write_to_file('AU2s OK.').

compliance_AU1S(MCTP_AU1S, PCTP_AU1S) :-
    check_length(MCTP_AU1S,PCTP_AU1S,'AU1s'),
    au1s(MCTP_AU1S,PCTP_AU1S, R),
    write('AU1s OK.'),nl,nl,
    write_to_file('AU1s OK.').

compliance_INSTRUMENTS(MCTP_INSTRUMENTS, PCTP_INSTRUMENTS) :-
    check_length(MCTP_INSTRUMENTS,PCTP_INSTRUMENTS,'instruments'),
    subset(MCTP_INSTRUMENTS, PCTP_INSTRUMENTS),
    write('Instruments OK.'),nl,nl,
    write_to_file('Instruments OK.').

% check AU1s

au1s([], _, []).
au1s([Single_MCTP_AU1|T], PCTP_AU1_List, Res) :-
    au1s_aux(Single_MCTP_AU1, PCTP_AU1_List, (Single_MCTP_AU1, Single_PCTP_AU1)),
    delete(PCTP_AU1_List, Single_PCTP_AU1, Removed),
    au1s(T, Removed, Sol),
    append([(Single_MCTP_AU1, Single_PCTP_AU1)], Sol, Res).

au1s_aux(Single_MCTP_AU1,[Single_PCTP_AU1|T],(Single_MCTP_AU1, Single_PCTP_AU1)) :-
    two_au1s(Single_MCTP_AU1, Single_PCTP_AU1, (Single_MCTP_AU1, Single_PCTP_AU1)),!.

au1s_aux(Single_MCTP_AU1,[Single_PCTP_AU1|T],R) :-
    au1s_aux(Single_MCTP_AU1,T,R).

two_au1s((AU1_A, AU1_A_connection), (AU1_B, AU1_B_connection), (Single_MCTP_AU1, Single_PCTP_AU1)) :-
    AU1_A = AU1_B.

% check AU2s

au2s([], _, []).
au2s([Single_MCTP_AU2|T], PCTP_AU2_List, Res) :-
    au2s_aux(Single_MCTP_AU2, PCTP_AU2_List, (Single_MCTP_AU2, Single_PCTP_AU2)),
    delete(PCTP_AU2_List, Single_PCTP_AU2, Removed),
    au2s(T, Removed, Sol),
    append([(Single_MCTP_AU2, Single_PCTP_AU2)], Sol, Res).

au2s_aux(Single_MCTP_AU2,[Single_PCTP_AU2|T],(Single_MCTP_AU2, Single_PCTP_AU2)) :-
    two_au2s(Single_MCTP_AU2, Single_PCTP_AU2, (Single_MCTP_AU2, Single_PCTP_AU2)),!.

au2s_aux(Single_MCTP_AU2,[Single_PCTP_AU2|T],R) :-
    au2s_aux(Single_MCTP_AU2,T,R).

two_au2s((AU2_A, connected_to(AU2_A_connection)), (AU2_B, connected_to(AU2_B_connection)), (Single_MCTP_AU2, Single_PCTP_AU2)) :-
    AU2_A = AU2_B,
    length(AU2_A_connection, A_Length),
    length(AU2_B_connection, B_Length),
    ignore((A_Length \= B_Length) ->  (write('AU2s connections are different '),nl,nl)),
    A_Length = B_Length.

